import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email import encoders
import os
from datetime import datetime

from rpanagem import manipula_diretorios_arquivos
import logging
logger = logging.getLogger(__name__)
log = manipula_diretorios_arquivos.Arquivos()
module_logger = log.criar_logger_diario("./log", nome_arquivo_log="enviar_email.log", logger=logger)


class ConetarSMTP():
    """realiza conexões smtp para enviou de email"""

    def __init__(self):
        pass

    def conectar_smtp_server(
            self,
            host,
            porta,
            usuario=None,
            senha=None,
            usar_ssl=False
    ):
        """
        Estabelece uma conexão com um servidor SMTP com ou sem SSL e, opcionalmente, realiza autenticação.

        :param host: Endereço do servidor SMTP.
        :type host: str
        :param porta: Porta do servidor SMTP
        :type porta: int
        :param usuario: (Opcional): Nome de usuário para autenticação SMTP.
        :type usuario: str
        :param senha: (Opcional): Senha correspondente ao usuário.
        :type senha: str
        :param usar_ssl: Define se a conexão deve ser feita via SSL.
        :type usar_ssl: bool

        :returns: Objeto de conexão SMTP pronto para uso, ou None em caso de erro.
        :rtype usar_ssl: smtplib.SMTP or smtplib.SMTP_SSL
        """
        try:
            if usar_ssl:
                conexao = smtplib.SMTP_SSL(host, porta)
            else:
                conexao = smtplib.SMTP(host, porta)
                conexao.ehlo()

            if usuario and senha:
                conexao.login(usuario, senha)

            # print("✅ Conexão com o servidor SMTP realizada com sucesso.")
            module_logger.info("✅ Conexão com o servidor SMTP realizada com sucesso.")

            return conexao
        except Exception as e:
            # print(f"❌ Erro ao conectar ao servidor SMTP: {e}")
            module_logger.error(f"❌ Erro ao conectar ao servidor SMTP: {e}")
            module_logger.error("❌ Verifique a conexão com a rede/VPN")

            return None

    def enviar_email(
            self,
            conexao,
            remetente="automation@nagem.com.br",
            destinatario="",
            cc=None,
            assunto="RPA",
            corpo="",
            corpo_html=False,
            dados_html={
                "MensagemEmail": "",
                },
            anexos=None
    ):
        """
        Envia um e-mail utilizando uma conexão SMTP existente, com suporte a corpo de texto e anexos.

        :param conexao: Objeto de conexão SMTP previamente estabelecido.
        :type conexao: smtplib.SMTP or smtplib.SMTP_SSL
        :param remetente: Endereço de e-mail do remetente.
        :type remetente: str
        :param destinatario: Endereço(s) de e-mail dos destinatários.
        :type destinatario: str ou list
        :param assunto: Assunto do e-mail.
        :type assunto: str
        :param corpo: Corpo do e-mail no formato texto simples ou um caminho para o arquivo .html
        :type corpo: str
        :param corpo_html: Define se o parametro corpo é um caminho para um arquivo .html.
        :type corpo_html: bool
        :param dados_html: Dicionário com os valores da chaves para serem substituidas no html.
        :type dados_html: str
        :param anexos: (Opcional) Caminho(s) para os arquivos a serem anexados ao e-mail.
        :type anexos: str ou list

        :returns: True se o e-mail for enviado com sucesso, False em caso de erro.
        :rtype usar_ssl: bool
        """
        try:
            if conexao is None:
                module_logger.warning("❌ Objeto de conexão SMTP é None. Verifique a conexão antes de enviar o e-mail.")
                raise ValueError("Objeto de conexão SMTP é None. Verifique a conexão antes de enviar o e-mail.")

            # Formatar destinatários
            if isinstance(destinatario, str):
                destinatario = [email.strip() for email in destinatario.split(",")]

            if cc:
                if isinstance(cc, str):
                    cc = [email.strip() for email in cc.split(",")]
            else:
                cc = []

            # Criar a mensagem
            mensagem = MIMEMultipart()
            mensagem["From"] = remetente
            mensagem["To"] = ", ".join(destinatario)
            if cc:
                mensagem["Cc"] = ", ".join(cc)
            mensagem["Subject"] = assunto

            # Anexar corpo em texto e/ou HTML
            if corpo_html:
                subtype = "html"

                # Lê o HTML do arquivo
                with open(corpo, "r", encoding="utf-8") as arquivo_html:
                    corpo = arquivo_html.read()

                # Dicionário com os dados para substituir os placeholders
                dados_html['DataEmail'] = datetime.now().strftime("%d/%m/%Y")

                # Substitui os placeholders no HTML
                for chave, valor in dados_html.items():
                    # substitui {{chave}} por valor
                    corpo = corpo.replace(f"{{{{{chave}}}}}", str(valor))
            else:
                subtype = "plain"

            mensagem.attach(MIMEText(corpo, subtype))

            # Adicionar anexos
            if anexos:
                if isinstance(anexos, str):
                    anexos = [anexos]  # Converte para lista se for string

                for caminho in anexos:
                    if not os.path.isfile(caminho):
                        # print(f"📣 Arquivo não encontrado: {caminho}")
                        module_logger.warning(f"📣 Arquivo não encontrado: {caminho}")
                        continue

                    try:
                        with open(caminho, "rb") as f:
                            parte = MIMEBase("application", "octet-stream")
                            parte.set_payload(f.read())
                            encoders.encode_base64(parte)
                            parte.add_header(
                                "Content-Disposition",
                                f'attachment; filename="{os.path.basename(caminho)}"'
                            )
                            mensagem.attach(parte)
                    except Exception as e:
                        module_logger.error(f"❌ Erro ao anexar arquivo {caminho}: {e}")
                        continue

            # Enviar e-mail
            todos_destinatarios = destinatario + cc
            conexao.sendmail(remetente, todos_destinatarios, mensagem.as_string())
            # conexao.quit()

            # print("✅ E-mail enviado com sucesso.")
            module_logger.info("✅ E-mail enviado com sucesso.")
            return True

        except Exception as e:
            # print(f"❌ Erro ao enviar o e-mail: {e}")
            module_logger.error(f"❌ Erro ao enviar o e-mail: {e}")
            return False
        finally:
            try:
                conexao.quit()
            except Exception:
                pass



import os
from openpyxl import Workbook
import logging
# import logging.handlers
from logging.handlers import TimedRotatingFileHandler

from openpyxl import load_workbook
from openpyxl.styles import PatternFill, Font, Alignment
from openpyxl.utils import get_column_letter
import pandas as pd
import numpy as np
import chardet
import shutil
from pathlib import Path
from typing import List, Optional, Dict, Any, Union
import datetime


class Arquivos():
    """Manipula arquivos."""

    def __init__(self):
        self.module_logger = self.criar_logger_diario("./log", nome_arquivo_log="manipula_diretorio_arquivos.log", logger=logging.getLogger(__name__))

    def arquivos_existem(self, *caminhos):
        """
        Verifica se todos os arquivos fornecidos existem.

        :param *caminhos: Um ou mais caminhos de arquivos a serem verificados.
        :type *caminhos: str

        :returns: True se todos os arquivos existem, False caso contrário.
        :rtype: bool
        """
        return all(os.path.isfile(caminho) for caminho in caminhos)

    def criar_arquivo(self, caminho):
        """
        Cria um arquivo vazio caso ele não exista no caminho especificado.

        Cria o(s) diretorio(s) caso não exista.

        :param caminho: Caminho completo do arquivo a ser criado.
        :type caminho: str

        :returns: True se o arquivo foi criado, False caso contrário.
        :rtype: bool
        """
        if not self.arquivos_existem(caminho):
            # Garante que o diretório do arquivo exista
            os.makedirs(os.path.dirname(caminho), exist_ok=True)
            with open(caminho, 'w') as f:
                pass  # Cria um arquivo vazio
            # print(f"✅ Arquivo criado: {caminho}")
            self.module_logger.info(f"✅ Arquivo criado: {caminho}")
            return True
        else:
            # print(f"ℹ️ Arquivo já existe: {caminho}")
            self.module_logger.warning(f"ℹ️ Arquivo já existe: {caminho}")
            return False

    def criar_xlsx_com_colunas(self, caminho_arquivo, colunas=None):
        """
        Cria um arquivo .xlsx com uma planilha contendo as colunas se forem especificadas.

        Sobrescreve o arquivo se ele já existir.

        :param caminho_arquivo: Caminho e nome do arquivo Excel a ser criado.
        :type caminho_arquivo: str
        :param colunas: Lista com os nomes das colunas. (ex: ["Nome", "Idade", "Email"]).
        :type colunas: list

        :returns: True se o caminho for criado com sucesso, False caso contrario
        :rtype: bool
        """
        if not self.arquivos_existem(caminho_arquivo):
            self.criar_arquivo(caminho_arquivo)
        try:
            wb = Workbook()
            ws = wb.active

            if colunas:
                ws.append(colunas)  # Insere os nomes das colunas como primeira linha

            wb.save(caminho_arquivo)
            # print(f"Arquivo '{caminho_arquivo}' criado com as colunas: {colunas}")
            self.module_logger.info(f"Arquivo '{caminho_arquivo}' criado com as colunas: {colunas}")
            return True
        except Exception as erro:
            # print('❌ OCORREU UM ERRO:', erro)
            self.module_logger.error(f"❌ OCORREU UM ERRO: {erro}")
            return False

    def inserir_tabela_xlsx(self, lista_de_dicionarios, caminho_arquivo, dic_nome_colunas=None):
        """
        Monta tabela e salva em uma planilha xlsx.

        :param lista_de_dicionarios: Lista contendo dicionários com os dados.
        :type lista_de_dicionarios: list
        :param caminho_arquivo: Caminho completo do arquivo Excel (.xlsx) a ser formatado.
        :type caminho_arquivo: str
        :param dic_nome_colunas: dicionario contendo as colunas de lista_de_dicionarios como chaves e o nome da coluna que deseja como valor.
        :type dic_nome_colunas: dict
        """
        try:
            self.module_logger.info("Estruturando dados em tabela...")
            df = pd.DataFrame(lista_de_dicionarios)

            # Converte campos datetime para string, se houver
            for col in df.columns:
                if df[col].dtype == 'datetime64[ns]' or df[col].apply(lambda x: str(type(x))).str.contains("datetime").any():
                    df[col] = df[col].dt.strftime('%d/%m/%Y')

            if dic_nome_colunas is not None:
                #  Apenas as colunas indicadas são renomeadas.
                # inplace=True altera o df diretamente.
                df.rename(columns = dic_nome_colunas,
                          inplace=True)

                # substituir todos os nomes das colunas:
                # df.columns = ['CPF', 'APELIDO', 'DATA HORA CRIAÇÃO', 'DATA HORA MODIFICAÇÃO', 'DATA ADMISSÃO', 'STAATV RM', 'NOME', 'LOG', 'CÓDIGO SAP']

            df.to_excel(caminho_arquivo, index=False)
            self.module_logger.info(f"✅ Relatório salvo com sucesso em: {caminho_arquivo}")

        except Exception as e:
            self.module_logger.error(f"❌ Erro ao estruturar dados em tabela: {e}")

    def formatar_xlsx(self, caminho_arquivo, aba="Sheet1"):
        """
        Aplica estilo visual ao cabeçalho de uma planilha Excel e ajusta automaticamente a largura das colunas.

        :param caminho_arquivo: Caminho completo do arquivo Excel (.xlsx) a ser formatado.
        :type caminho_arquivo: str
        :param aba: Nome da aba onde aplicar o estilo. Padrão é 'Sheet1'.
        :type aba: str

        Estilos aplicados:
        - Cabeçalho com fundo azul (cor: #0070C0)
        - Texto do cabeçalho em branco, negrito e centralizado
        - Largura das colunas ajustada com base no maior conteúdo da coluna
        """
        try:
            self.module_logger.info("Formatando o estilo do relatorio...")

            wb = load_workbook(caminho_arquivo)

            try:
                ws = wb[aba]
            except:
                ws = wb["Sheet"]

            # Definição do estilo azul
            cabecalho_fill = PatternFill(start_color="0070C0", end_color="0070C0", fill_type="solid")
            fonte_branca = Font(color="FFFFFF", bold=True)
            alinhamento_central = Alignment(horizontal="center", vertical="center")

            for cell in ws[1]:  # Primeira linha = cabeçalho
                cell.fill = cabecalho_fill
                cell.font = fonte_branca
                cell.alignment = alinhamento_central

            wb.save(caminho_arquivo)
            self.module_logger.info("✅ Formatação aplicada")

        except Exception as e:
            self.module_logger.error(f"❌ Erro ao formatar o estilo do relatorio: {e}")


        try:
            self.module_logger.info("Ajustando a largura das colunas do relatorio...")

            wb = load_workbook(caminho_arquivo)
            try:
                ws = wb[aba]
            except:
                ws = wb["Sheet"]

            for col in ws.columns:
                max_length = 0
                col_letter = get_column_letter(col[0].column)
                for cell in col:
                    try:
                        if cell.value:
                            max_length = max(max_length, len(str(cell.value)))
                    except:
                        pass
                ws.column_dimensions[col_letter].width = max_length + 4  # margem extra

            wb.save(caminho_arquivo)

            self.module_logger.info("✅ Estilo e largura aplicados")
        except Exception as e:
            self.module_logger.error(f"❌ Erro ao aplicar largura: {e}")

        self.module_logger.info(f"✅ Relatório formatado e salvo com sucesso em: {caminho_arquivo}")

    def criar_logger_diario(self, diretorio_log, nome_arquivo_log="log", logger=logging.getLogger()):
        """
        Cria e configura um logger que grava logs com rotação diária, criando um novo arquivo a cada dia.

        :param diretorio_log: Caminho para o diretório onde os logs serão salvos.
        :type diretorio_log: str
        :param nome_arquivo_log: Nome base do arquivo de log (ex: "log" -> log_2025-06-20.log).
        :type nome_arquivo_log: str
        :param logger: Logger nomeado para __name__.
        :type logger: logging.Logger

        :returns: Logger configurado com rotação diária.
        :rtype: logging.Logger
        """
        # Caminho base para logs rotacionados: log/log_
        dir_arq_log = os.path.join(diretorio_log, nome_arquivo_log)

        try:
            # self.criar_arquivo(dir_arq_log)
            os.makedirs(diretorio_log, exist_ok=True)

            logger.setLevel(logging.DEBUG)
        except Exception as e:
            raise Exception(f"❌ Rootlogger não pode ser iniciado.: {e}")

        try:
            if not logger.handlers:
                formatter = logging.Formatter(
                    "%(name)s - %(levelname)s - %(asctime)s - %(message)s"
                )

                # Handler com rotação diária
                handler = TimedRotatingFileHandler(
                    filename=dir_arq_log,
                    when='midnight',             # rotação diária
                    interval=1,                  # a cada 1 dia
                    backupCount=30,              # mantém últimos 30 arquivos
                    encoding='utf-8',
                    utc=False
                )
                handler.suffix = "%Y-%m-%d.log"  # formato do nome final do arquivo
                handler.setFormatter(formatter)

                # Também loga no console
                console_handler = logging.StreamHandler()
                console_handler.setFormatter(formatter)

                logger.addHandler(handler)
                logger.addHandler(console_handler)

            return logger
        except Exception as e:
            raise Exception(f"O handler do arquivo de log não pode ser configurado. {e}")

    def mover_arquivo(self, origem, destino_pasta, sobrescrever=False):
        """Move um arquivo para outro diretorio.

        :param origem: Caminho completo do arquivo.
        :type origem: str
        :param destino_pasta: Caminho para onde se quer mover o arquivo
        :type destino_pasta: str
        :param sobrescrever: Define se deve sobrescrever um arquivo existente no destino.
        :type sobrescrever: bool
        :raises FileNotFoundError: Se o arquivo de origem ou o diretório de destino não forem encontrados.
        """
        nome_arquivo = os.path.basename(origem)
        destino_arquivo = os.path.join(destino_pasta, nome_arquivo)


        if not self.arquivos_existem(origem):
            self.module_logger.warning("❌ Arquivo não encontrado, verifique se o caminho está correto.")
            raise FileNotFoundError("❌ Arquivo não encontrado, verifique se o caminho está correto.")

        if not os.path.isdir(destino_pasta):
            self.module_logger.warning("❌ Diretório de destino não encontrado.")
            raise FileNotFoundError("❌ Diretório de destino não encontrado.")

        if os.path.exists(destino_arquivo):
            if not sobrescrever:
                self.module_logger.warning("⚠️ Já existe um arquivo com o mesmo nome no destino.")
                raise FileExistsError("⚠️ Já existe um arquivo com o mesmo nome no destino.")
            else:
                self.module_logger.info("♻️ Arquivo existente será sobrescrito.")

        shutil.move(origem, destino_arquivo)
        self.module_logger.info(f"✅ Arquivo movido com sucesso para {destino_arquivo}")

    def copiar_arquivo(self, origem, destino_pasta, sobrescrever=False):
        """
        Copia um arquivo para outro diretório.

        :param origem: Caminho completo do arquivo de origem.
        :type origem: str
        :param destino_pasta: Caminho para o diretório de destino.
        :type destino_pasta: str
        :param sobrescrever: Define se deve sobrescrever um arquivo existente no destino.
        :type sobrescrever: bool
        :raises FileNotFoundError: Se o arquivo de origem ou o diretório de destino não forem encontrados.
        """
        nome_arquivo = os.path.basename(origem)
        destino_arquivo = os.path.join(destino_pasta, nome_arquivo)

        if not self.arquivos_existem(origem):
            self.module_logger.warning("❌ Arquivo não encontrado, verifique se o caminho está correto.")
            raise FileNotFoundError("❌ Arquivo não encontrado, verifique se o caminho está correto.")

        if not os.path.isdir(destino_pasta):
            self.module_logger.warning("❌ Diretório de destino não encontrado.")
            raise FileNotFoundError("❌ Diretório de destino não encontrado.")

        if os.path.exists(destino_arquivo):
            if not sobrescrever:
                self.module_logger.warning("⚠️ Já existe um arquivo com o mesmo nome no destino.")
                raise FileExistsError("⚠️ Já existe um arquivo com o mesmo nome no destino.")
            else:
                self.module_logger.info("♻️ Arquivo existente será sobrescrito.")

        shutil.copy2(origem, destino_arquivo)
        self.module_logger.info(f"✅ Arquivo copiado com sucesso para {destino_arquivo}")
    
    def detectar_encoding(self, arquivo_csv, num_bytes=1024):
        """
        Detecta o encoding (codificação de caracteres) de um arquivo de texto ou CSV.

        Esta função utiliza a biblioteca `chardet` para analisar os primeiros bytes do arquivo
        e estimar a codificação mais provável (como 'utf-8', 'ISO-8859-1', 'cp1252', etc.).
        É útil para evitar erros de decodificação ao ler arquivos com `pandas` ou `open()`.

        :param arquivo_csv: Caminho para o arquivo que será analisado.
        :type arquivo_csv: str
        :param num_bytes: Quantidade de bytes lidos do início do arquivo para análise (padrão: 1024).
        :type num_bytes: int

        :return: O encoding detectado (ex: 'utf-8', 'latin1').
        :rtype: str
        """
        with open(arquivo_csv, 'rb') as f:
            resultado = chardet.detect(f.read(num_bytes))
            return resultado['encoding']

    def ler_csv(self, arquivo_csv,
                linha_cabecalho: Union[int, None] = 0,
                linha_inicio=0,
                colunas_desejadas=None,
                tipo_colunas=None,
                delimiter=';',
                quantidade_linhas_lidas=None,
                strict: bool = True,
                encoding=None,):
        """
        Lê um CSV usando pandas com suporte a encoding, linha inicial, colunas filtradas,
        delimitador customizado e quantidade limitada de linhas.

        :param arquivo_csv: Caminho do CSV.
        :type arquivo_csv: str
        :param linha_cabecalho: Número da linha usada como cabeçalho (default=0).
        :type linha_cabecalho: int or None
        :param linha_inicio: Quantidade de linhas para 'pular'.
        :type linha_inicio: int
        :param colunas_desejadas: Lista de colunas a manter. Se None, retorna todas.
        :type colunas_desejadas: list or None
        :param tipo_colunas: Dict mapeando colunas para tipos desejados (dtype do pandas).
        :type tipo_colunas: dict
        :param delimiter: Separador do CSV. Padrão: ';'.
        :type delimiter: str
        :param quantidade_linhas_lidas: Número máximo de linhas de dados a ler (ignora após esse limite).
        :type quantidade_linhas_lidas: int or None
        :param strict: Se True, lança erro caso colunas_desejadas não existam.
        :type strict: bool
        :param encoding: Codificação de caracteres.
        :type encoding: str

        :return: Lista de dicionários (cada linha como dict).
        :rtype: list
        """
        if linha_inicio < 0:
            raise ValueError("⚠️ 'linha_inicio' deve ser maior ou igual 0.")
        if quantidade_linhas_lidas is not None and quantidade_linhas_lidas <= 0:
            raise ValueError("⚠️ 'quantidade_linhas_lidas' deve ser maior que 0.")

        try:
            # Pula linhas apenas depois do header
            if linha_cabecalho is None:
                skiprows = linha_inicio if linha_inicio > 0 else None
            else:
                skiprows = range(linha_cabecalho + 1, linha_cabecalho + 1 + linha_inicio) if linha_inicio > 0 else None

            if not encoding:
                encoding = self.detectar_encoding(arquivo_csv)

            # pandas assume que a primeira linha tem os nomes das colunas
            df = pd.read_csv(arquivo_csv,
                             delimiter=delimiter,
                             header=linha_cabecalho,
                             dtype=tipo_colunas,
                             skiprows=skiprows,  # Pula as primeiras n linhas do arquivo ao carregá-lo.
                            #  decimal=","
                             nrows=quantidade_linhas_lidas,
                             encoding=encoding,
                             )

            # Remove linhas onde todos os valores são NaN
            df = df.dropna(how='all')

            # substituindo valores None para Nan e em seguida substituindo Nan para vazio
            df = df.replace([None], np.nan).fillna("")

            # print(df.head(5))
            # print(df.shape[0])  # quantidade de linhas (sem cabeçalho)
            # print(df.shape[1])  # quantidade de colunas
            
            if colunas_desejadas:
                colunas_invalidas = [c for c in colunas_desejadas if c not in df.columns]
                if colunas_invalidas:
                    msg = f"⚠️ Colunas não encontradas no CSV: {colunas_invalidas}"
                    self.module_logger.warning(msg)
                    if strict:
                        raise KeyError(msg)
                    else:
                        colunas_desejadas = [c for c in colunas_desejadas if c in df.columns]
                df = df[colunas_desejadas]
            
            self.module_logger.info(f"CSV carregado com {df.shape[0]} linhas e {df.shape[1]} colunas.")

            # print(df.head(5).to_dict(orient='dict'))  # {'CONTRATO': [...], 'MATRICULA': [...]}
            # print(df.head(5).to_dict(orient='list'))  # mesmo que acima, mas com listas ordenadas
            # print(df.head(5).to_dict(orient='series'))  # valores como pd.Series
            return df.to_dict(orient='records')

        except FileNotFoundError:
            erro = f"❌ Arquivo '{arquivo_csv}' não foi encontrado!"
            self.module_logger.error(erro)
            raise
        except Exception as e:
            self.module_logger.error(f"❌ Erro ao ler CSV '{arquivo_csv}': {e}")
            raise
    
    def obter_arquivos(self, diretorio: str, extensao: Optional[str] = None) -> List[str]:
        """
        Lista caminhos absolutos de arquivos diretamente contidos no diretório especificado,
        com suporte a filtro por extensão (sem buscar em subpastas).

        :param diretorio: Caminho do diretório base.
        :type diretorio: str
        :param extensao: (Optional) Ex.: ".txt" para filtrar arquivos por extensão.
        :type extensao: str

        :return: Lista de caminhos absolutos dos arquivos encontrados.
        :rtype: List[str]

        :raises: ValueError Se o caminho não existir ou não for um diretório.
        """
        path = Path(diretorio)

        self.module_logger.info(f"Verificando diretório: {diretorio}")
        if not path.exists():
            self.module_logger.info(f"⚠️ O diretório '{diretorio}' não existe.")
            raise ValueError(f"⚠️ O diretório '{diretorio}' não existe.")
        if not path.is_dir():
            self.module_logger.info(f"⚠️ O caminho '{diretorio}' não é um diretório.")
            raise ValueError(f"⚠️ O caminho '{diretorio}' não é um diretório.")

        self.module_logger.info("Iniciando varredura...")
        arquivos_encontrados = []

        try:
            # for arquivo in path.rglob('*'):  # obtem tudo inclusive subpastas
            for arquivo in path.iterdir():  # somente o conteúdo direto do diretório
                if not arquivo.is_file():
                    continue
                if extensao and arquivo.suffix.lower() != extensao.lower():
                    continue
                try:
                    caminho_resolvido = str(arquivo.resolve(strict=False))
                    arquivos_encontrados.append(caminho_resolvido)
                    self.module_logger.info(f"Arquivo encontrado: {caminho_resolvido}")
                except Exception as e:
                    self.module_logger.error(f"❌ Erro ao resolver caminho de '{arquivo}': {e}")
        except Exception as e:
            self.module_logger.error(f"❌ Erro durante a varredura de arquivos: {e}.")
            raise Exception(f"❌ Erro durante a varredura de arquivos: {e}")

        self.module_logger.info(f"Total de arquivos encontrados: {len(arquivos_encontrados)}")
        return arquivos_encontrados

    def ler_txt(self, arquivo_txt, encoding='utf-8', remover_quebra_linha=True):
        """
        Lê um arquivo .txt e retorna uma lista com as linhas do arquivo.

        :param caminho_arquivo_txt: Caminho completo ou relativo para o arquivo .txt
        :type caminho_arquivo_txt: str
        :param encoding: Codificação do arquivo (default: utf-8)
        :type encoding: str
        :param remover_quebra_linha: Se True, remove os \n das linhas
        :type remover_quebra_linha: bool

        :return: Lista de linhas (strings)
        :rtype: list

        :raises: IOError em caso de erro de leitura
        """
        encoding = self.detectar_encoding(arquivo_txt)

        try:
            with open(arquivo_txt, 'r', encoding=encoding) as arquivo:
                if remover_quebra_linha:
                    return [linha.rstrip('\n') for linha in arquivo]
                else:
                    return arquivo.readlines()
        except Exception as e:
            self.module_logger.error(f"❌ Erro ao ler o arquivo '{arquivo_txt}': {e}")
            raise IOError(f"❌ Erro ao ler o arquivo '{arquivo_txt}': {e}")

    def obter_infomacoes_arquivo(self, caminho_arquivo):
        """
        Retorna informações detalhadas sobre um arquivo.
        
        :param caminho_arquivo: Caminho completo do arquivo
        :type caminho_arquivo: str

        :return: dicionário com informações extraídas
        :rtype: dict

        :raises FileNotFoundError: se o arquivo não existir
        """
        caminho = Path(caminho_arquivo)

        if not caminho.is_file():
            self.module_logger.error(f"❌ Arquivo não encontrado: {caminho_arquivo}")
            raise FileNotFoundError(f"❌ Arquivo não encontrado: {caminho_arquivo}")

        stat = caminho.stat()

        return {
            "data_criacao": datetime.datetime.fromtimestamp(stat.st_ctime),
            "data_ultima_gravacao": datetime.datetime.fromtimestamp(stat.st_mtime),
            "nome_com_extensao": caminho.name,
            "nome_sem_extensao": caminho.stem,
            "nome_completo": str(caminho.resolve()),
            "extensao": caminho.suffix.lstrip('.'),
            "diretorio": str(caminho.parent),
            "tamanho_bytes": stat.st_size,
        }

    def alterar_valor_celula_vazia_xlsx(self, caminho_arquivo, dic_coluna_valor):
        """
        Altera a celula de determinada coluna quando o valor é vazio
        
        :param caminho_arquivo: Caminho completo do arquivo
        :type caminho_arquivo: str

        :param dic_coluna_valor: Dicionario com a coluna que se deseja alterar como chave e o valor que se deseja adicionar como valor
        :type dic_coluna_valor: dict
        """
        try:
            # Carregar o arquivo Excel
            df = pd.read_excel(caminho_arquivo)

            # Substituir valores vazios na coluna
            for coluna, valor in dic_coluna_valor.items():
                df[coluna] = df[coluna].fillna(valor)

            # Substituir valores 2 por 'LANÇADO' na coluna 'Status'
            # df['Status'] = df['Status'].replace('LANÇADO', 'TESTE')

            # Salvar o resultado em um novo arquivo
            df.to_excel(caminho_arquivo, index=False)
            self.module_logger.error(f"✅ Valores das colunas {dic_coluna_valor.keys()} alteradas")
        except Exception as erro:
            self.module_logger.error(f"❌ Erro ao alterar o valor das colunas vazia: {erro}")
            raise Exception(f"❌ Erro ao alterar o valor da coluna vazia: {erro}")
    
    def alterar_valor_celula_xlsx_com_condicao(self, caminho_arquivo, list_coluna_valores):
        """
        Altera o valor da celula se a propria tiver um valor específico.

        :param caminho_arquivo: Caminho completo do arquivo
        :type caminho_arquivo: str

        :param list_coluna_valores: Uma lista com os 3 item obrigatorio [coluna, valor_atual, valor_alterado] respctivamente nessa ordem.
        :type list_coluna_valores: list
        """
        try:
            # Carregar o arquivo Excel
            df = pd.read_excel(caminho_arquivo)

            coluna = list_coluna_valores[0]
            valor_atual = str(list_coluna_valores[1])
            valor_alterado = str(list_coluna_valores[2])

            # Substituir valores
            df[coluna] = df[coluna].replace(valor_atual, valor_alterado)

            # Salvar o resultado em um novo arquivo
            df.to_excel(caminho_arquivo, index=False)
            self.module_logger.error(f"✅ Coluna: {coluna} | Valor atual: {valor_atual} | Alterado para: {valor_alterado}")
        except Exception as erro:
            self.module_logger.error(f"❌ Erro ao alterar o valor da coluna: {erro}")
            raise Exception(f"❌ Erro ao alterar o valor da coluna: {erro}")

    def ler_xlsx(self,
                arquivo_xlsx: str,
                sheet_name: str = 0,
                linha_cabecalho: Union[int, None] = 0,
                linha_inicio: int = 0,
                colunas_desejadas: Optional[List[str]] = None,
                tipo_colunas: Optional[dict] = None,
                quantidade_linhas_lidas: Optional[int] = None,
                strict: bool = True) -> List[Dict[str, Any]]:
        """
        Lê um arquivo Excel (.xlsx) usando pandas com suporte a filtro de colunas,
        tipos de dados, linha inicial e quantidade máxima de linhas.

        :param arquivo_xlsx: Caminho do arquivo Excel.
        :type arquivo_xlsx: str
        :param sheet_name: Nome ou índice da planilha (default=0, primeira planilha).
        :type sheet_name: str or int
        :param linha_cabecalho: Número da linha usada como cabeçalho (default=0).
        :type linha_cabecalho: int or None
        :param linha_inicio: Número de linhas a pular no início (default=0).
        :type linha_inicio: int
        :param colunas_desejadas: Lista de colunas a manter (ou None para manter todas).
        :type colunas_desejadas: list or None
        :param tipo_colunas: Dict mapeando colunas para tipos desejados (dtype do pandas).
        :type tipo_colunas: dict
        :param quantidade_linhas_lidas: Número máximo de linhas de dados a ler.
        :type quantidade_linhas_lidas: int
        :param strict: Se True, lança erro caso colunas_desejadas não existam.
        :type strict: bool

        :return: Lista de dicionários (cada linha como dict).
        :rtype: list
        """

        if linha_inicio < 0:
            raise ValueError("⚠️ 'linha_inicio' deve ser maior ou igual 0.")
        if quantidade_linhas_lidas is not None and quantidade_linhas_lidas <= 0:
            raise ValueError("⚠️ 'quantidade_linhas_lidas' deve ser maior que 0.")

        try:
            # Pula linhas apenas depois do header
            if linha_cabecalho is None:
                skiprows = linha_inicio if linha_inicio > 0 else None
            else:
                skiprows = range(linha_cabecalho + 1, linha_cabecalho + 1 + linha_inicio) if linha_inicio > 0 else None

            df = pd.read_excel(
                arquivo_xlsx,
                sheet_name=sheet_name,
                header=linha_cabecalho,
                dtype=tipo_colunas,
                skiprows=skiprows,
                nrows=quantidade_linhas_lidas,
                engine="openpyxl"  # engine padrão para .xlsx
            )

            df = df.dropna(how="all").fillna("")

            if colunas_desejadas:
                colunas_invalidas = [c for c in colunas_desejadas if c not in df.columns]
                if colunas_invalidas:
                    msg = f"⚠️ Colunas não encontradas no XLSX: {colunas_invalidas}"
                    self.module_logger.warning(msg)
                    if strict:
                        raise KeyError(msg)
                    else:
                        colunas_desejadas = [c for c in colunas_desejadas if c in df.columns]
                df = df[colunas_desejadas]

            self.module_logger.info(
                f"Excel carregado da planilha '{sheet_name}' com {df.shape[0]} linhas e {df.shape[1]} colunas."
            )

            return df.to_dict(orient="records")

        except FileNotFoundError:
            erro = f"❌ Arquivo '{arquivo_xlsx}' não foi encontrado!"
            self.module_logger.error(erro)
            raise

        except Exception as e:
            self.module_logger.error(f"❌ Erro ao ler XLSX '{arquivo_xlsx}': {e}")
            raise



class Diretorios():
    """Manipula diretórios."""

    def __init__(self):
        obj_arq = Arquivos()
        self.module_logger = obj_arq.criar_logger_diario("./log", nome_arquivo_log="manipula_diretorio_arquivos.log", logger=logging.getLogger(__name__))

    def diretorio_existe(self, caminho):
        """
        Verifica se um diretório existe no caminho especificado.

        :param caminho: Caminho do diretório a ser verificado.
        :type caminho: str

        :returns: True se o diretório existe, False caso contrário.
        :rtype: bool
        """
        return os.path.isdir(caminho)

    def criar_diretorio(self, caminho):
        """
        Cria o diretório especificado caso ele não exista.

        :param caminho: Caminho do diretório a ser verificado ou criado.
        :type caminho: str

        :returns: True se o diretório foi criado, False caso contrário.
        :rtype: bool
        """
        try:
            if not self.diretorio_existe(caminho):
                os.makedirs(caminho)
                # print(f"✅ Diretório criado: {caminho}")
                self.module_logger.info(f"✅ Diretório criado: {caminho}")
                return True
            else:
                # print(f"ℹ️ Diretório já existe: {caminho}")
                self.module_logger.warning(f"ℹ️ Diretório já existe: {caminho}")
                return False
        except OSError as erro:
            self.module_logger.warning(f"❌ {erro}")
            raise OSError(f"❌ {erro}")
        except Exception as erro:
            self.module_logger.warning(f"❌ {erro}")
            raise Exception(f"❌ {erro}")



from datetime import datetime, timedelta, date

from rpanagem import manipula_diretorios_arquivos
import logging
logger = logging.getLogger(__name__)
log = manipula_diretorios_arquivos.Arquivos()
module_logger = log.criar_logger_diario("./log", nome_arquivo_log="manipula_datahora.log", logger=logger)


# Mapeamento fixo (sem depender do sistema)
DIAS_PT = [
    "segunda-feira",
    "terça-feira",
    "quarta-feira",
    "quinta-feira",
    "sexta-feira",
    "sábado",
    "domingo"
]


class DataHora():
    """Manupila datas."""

    def __init__(self):
        pass

    def obter_incluir_valor_data(self, dias_para_subtrair=0, dias_para_adicionar=0, formato="%Y-%m-%d", datahora=False):
        """
        Retorna a data atual ajustada com base nos dias a subtrair e adicionar.

        Pode ser formatada conforme especificado.

        Caso nenhum parametro seja passado o retorno será a data atual no formato americano.

        :param dias_para_subtrair: Número de dias a subtrair da data atual.
        :type dias_para_subtrair: int
        :param dias_para_adicionar: Número de dias a adicionar à data resultante.
        :type dias_para_adicionar: int
        :param formato: Formato da data de retorno apenas para retorno str (ex: "%d/%m/%Y" ou "%Y-%m-%d").
        :type formato: str
        :param datahora: True para ser retornado data e hora no formato datetime.datetime, False retorna apenas a data no formato str (Padrão: False)
        :type datahora: bool

        :returns: A nova data ajustada no formato especificado caso o retorno seja str.
        :rtype: str
        """
        if datahora is True:
            data_atual = datetime.today()
            nova_data = data_atual - timedelta(days=dias_para_subtrair) + timedelta(days=dias_para_adicionar)
        elif datahora is False:
            data_atual = datetime.today().date()
            nova_data = (data_atual - timedelta(days=dias_para_subtrair) + timedelta(days=dias_para_adicionar)).strftime(formato)
        else:
            raise ValueError("❌ Paramentro datahora inválido. Use: True ou False.")

        return nova_data

    def converter_data(self, data=None, tipo_desejado="datetime", formato_str="%Y-%m-%d"):
        """
        Converte uma data recebida para o tipo especificado.

        :param data: A data de entrada.
        :type data: str, datetime, date ou timestamp
        :param tipo_desejado: Tipo de saída desejado. Pode ser:
            - "datetime"    2025-06-05 00:00:00
            - "date"    2025-06-05
            - "str" 2025-06-05
            - "timestamp"   1749092400
        :type tipo_desejado: str
        :param formato_str: formato que deseja ser retornado (Obrigatorio apenas se tipo_desejado for str).
        :type formato_str: str

        Retorna:
        - A data convertida no tipo especificado.
        """
        if not data:  # Trata None, '', 0
            if tipo_desejado == "str":
                return ""
            return None

        # Normaliza para datetime primeiro
        if isinstance(data, str):
            formatos = ["%Y-%m-%d", "%d/%m/%Y", "%d-%m-%Y", "%Y/%m/%d", "%d/%m/%Y %H:%M:%S", "%d-%m-%Y %H:%M:%S", "%Y-%m-%d %H:%M:%S", "%Y/%m/%d %H:%M:%S", "%Y%m%d", "%d%m%Y", "%d%m%Y", "%Y%m%d"]
            for fmt in formatos:
                try:
                    data = datetime.strptime(data, fmt)
                    break
                except ValueError:
                    continue
            else:
                raise ValueError(f"Formato de data inválido: {data}")

        elif isinstance(data, date) and not isinstance(data, datetime):
            data = datetime.combine(data, datetime.min.time())
        elif isinstance(data, (int, float)):
            data = datetime.fromtimestamp(data)

        # Converte para o tipo desejado
        if tipo_desejado == "datetime":
            return data
        elif tipo_desejado == "date":
            return data.date()
        elif tipo_desejado == "str":
            return data.strftime(formato_str)
        elif tipo_desejado == "timestamp":
            return int(data.timestamp())
        else:
            raise ValueError("❌ Tipo desejado inválido. Use: 'datetime', 'date', 'str', 'timestamp'.")

    def data_hora_para_binario(self, data_hora: datetime) -> str:
        """
        Converte uma data e hora para uma string binária.

        :param data_hora: Data hora para ser convertida.
        :type data_hora: datetime

        :returns: String binária representando cada parte da data e hora.
        :rtype: str
        """
        # Converter cada parte para binário com preenchimento para garantir o número de bits
        bin_ano = format(data_hora.year, '016b') # 16 bits
        bin_mes = format(data_hora.month, '04b') # 4 bits
        bin_dia = format(data_hora.day, '05b') # 5 bits
        bin_hora = format(data_hora.hour, '05b') # 5 bits
        bin_minuto = format(data_hora.minute, '06b') # 6 bits
        bin_segundo = format(data_hora.second, '06b') # 6 bits

        # Juntar tudo
        binario_completo = f"{bin_ano}{bin_mes}{bin_dia}{bin_hora}{bin_minuto}{bin_segundo}"
        return binario_completo

    def binario_para_data_hora(self, binario: str) -> datetime:
        """
        Converte uma string binária de volta para um objeto datetime.

        :param data_hora: Espera uma string de 42 bits no seguinte formato:
        [16 bits ano][4 bits mês][5 bits dia][5 bits hora][6 bits minuto][6 bits segundo]
        :type data_hora: str

        :returns: Data e hora.
        :rtype: datetime
        """
        if len(binario) != 42:
            raise ValueError("A string binária deve conter exatamente 42 bits.")

        # Separar os campos com base nos tamanhos definidos
        ano = int(binario[0:16], 2)
        mes = int(binario[16:20], 2)
        dia = int(binario[20:25], 2)
        hora = int(binario[25:30], 2)
        minuto = int(binario[30:36], 2)
        segundo = int(binario[36:42], 2)

        # Criar e retornar o objeto datetime
        return datetime(ano, mes, dia, hora, minuto, segundo)

    def obter_ultimo_dia_do_mes(self, formato="%d"):
        """
        Retorna o último dia do mês atual, formatado como string.

        :param formato: Formato da data como string (ex: "%d/%m/%Y", "%Y-%m-%d").
                        Usa o padrão ISO "%Y-%m-%d" se não for especificado.
        :type formato: str

        :returns: Último dia do mês atual formatado ou None em caso de erro
        :rtype: str
        """
        try:
            hoje = date.today()

            if hoje.month == 12:
                proximo_mes = hoje.replace(year=hoje.year + 1, month=1, day=1)
            else:
                proximo_mes = hoje.replace(month=hoje.month + 1, day=1)

            ultimo_dia = proximo_mes - timedelta(days=1)

            return ultimo_dia.strftime(formato)
        except Exception as erro:
            module_logger.error(f"❌ Erro ao obter o ultimo de do mês atual: {erro}")
            return None

    def obter_dia_da_semana(self, data: str | None = None) -> str:
        """
        Retorna o dia da semana em português, sem depender do locale do sistema.

        Args:
            data (str | None): Data no formato 'YYYY-MM-DD'.
                            Se None, usa a data de hoje.

        Returns:
            str: Nome do dia da semana em português.
        """
        if data:
            dt = datetime.strptime(data, "%Y-%m-%d").date()
        else:
            dt = date.today()

        return DIAS_PT[dt.weekday()]





# # Exemplos de uso:

# from rpanagem import manipula_datahora

# obj = manipula_datahora.DataHora()

# print(obj.obter_dia_da_semana())  # hoje → "segunda-feira" (se hoje for segunda)s
# print(obj.obter_dia_da_semana("2025-08-30"))  # "sábado"

# # Formato padrão (YYYY-MM-DD)
# print(obj.obter_incluir_valor_data(2, 3))

# # Formato brasileiro (DD/MM/YYYY)
# print(obj.obter_incluir_valor_data(5, 0, formato="%d/%m/%Y"))

# # Apenas mês e ano
# print(obj.obter_incluir_valor_data(0, 0, formato="%m/%Y"))

# # Obtendo data atual no formato brasileiro
# print(obj.obter_incluir_valor_data(formato="%d/%m/%Y"))

# print('#'*50)


# print(obj.converter_data("05/06/2025", "datetime")) # 2025-06-05 00:00:00
# print(obj.converter_data("05/06/2025", "timestamp")) # 1749092400
# print(obj.converter_data("05/06/2025", "date")) # 2025-06-05
# print(obj.converter_data("2025/06/25", "str", "%d-%m-%Y")) # 25-06-2025
# print(obj.converter_data(date.today(), "timestamp"))  # -> 1749524400
# print(obj.converter_data(datetime.now(), "str"))      # -> "2025-06-05"
# print(obj.converter_data(1728172800, "date"))         # -> datetime.date(2024, 10, 6)

# print('#'*50)


# data_exemplo = obj.obter_incluir_valor_data(datahora=True)
# resultado_binario = obj.data_hora_para_binario(data_exemplo)
# print("Data/hora em binário:", resultado_binario)

# data_recuperada = obj.binario_para_data_hora(resultado_binario)
# print("Data/hora reconstruída:", data_recuperada)




import pyodbc

from rpanagem import manipula_diretorios_arquivos
import logging
logger = logging.getLogger(__name__)
log = manipula_diretorios_arquivos.Arquivos()
module_logger = log.criar_logger_diario("./log", nome_arquivo_log="conexao_banco.log", logger=logger)


class ODBC():
    """Classe para conexõe com bancos de dados e execuções de qurerys"""

    def __init__(self):
        pass

    def conectar_odbc(self, driver, system, usuario, senha, database):
        """
        Cria uma conexão ODBC com um banco de dados.

        :param driver: Nome do driver ODBC instalado (ex: 'ODBC Driver 17 for SQL Server')
        :type driver: str
        :param system: Nome ou IP do servidor do banco de dados
        :type system: str
        :param usuario: Nome do usuário
        :type usuario: str
        :param senha: Senha do usuário
        :type senha: str
        :param database: Nome do banco de dados
        :type database: str

        :returns: Objeto de conexão pyodbc
        :rtype: pyodbc.Connection
        """
        try:
            conexao = pyodbc.connect(f"Driver={driver};"
                                    f"System={system};"
                                    "Uid=" + f'{usuario}' +
                                    ";Pwd=" + f'{senha}' +
                                    f";DefaultLibraries={database}")

            # print("✅ Conexão ODBC bem-sucedida!")
            module_logger.info("✅ Conexão ODBC bem-sucedida!")
            return conexao
        except pyodbc.Error as e:
            # print("❌ Erro ao conectar ao banco:", e)
            module_logger.error(f"❌ Erro ao conectar ao banco: {e}")
            module_logger.error("❌ Verifique a conexão com a rede/VPN")
            raise pyodbc.Error(f"❌ Erro ao conectar ao odbc: {e}")

    def executar_select(self, conexao, query):
        """
        Executa uma query SQL usando uma conexão ODBC e retorna os resultados como uma lista de dicionários.

        :param conexao: Objeto de conexao
        :type conexao: pyodbc.Connection
        :param query: Query SQL
        :type query: str

        :returns: Lista de dicionários (cada dicionário representa uma linha com os nomes das colunas como chaves)
        :rtype: list
        """
        try:
            cursor = conexao.cursor()
            cursor.execute(query)

            # Pega os nomes das colunas
            colunas = [desc[0] for desc in cursor.description]

            # Constrói lista de dicionários
            resultados = [dict(zip(colunas, linha)) for linha in cursor.fetchall()]

            for registro in resultados:
                for chave, valor in registro.items():
                    if valor is None:
                        registro[chave] = ""

            module_logger.info("✅ SELECT executado com sucesso.")

            return resultados

        except Exception as e:
            # print(f"❌ Erro ao executar a query: {e}")
            module_logger.error(f"❌ Erro ao executar a query SELECT: {e}")
            raise pyodbc.Error(f"❌ Erro ao executar a query SELECT: {e}")

    def conectar_sqlserver(self, driver, server, database, usuario, senha):
        """
        Função para conectar a um banco SQL Server.

        :param driver: Nome do driver ODBC instalado (ex: 'ODBC Driver 17 for SQL Server')
        :type driver: str
        :param server: endereço do servidor
        :type server: str
        :param database: nome do banco de dados
        :type database: str
        :param usuario: usuário do banco
        :type usuario: str
        :param senha: senha do banco
        :type senha: str

        :returns: Objeto de conexão pyodbc
        :rtype: pyodbc.Connection
        """
        try:
            conn_str = f'DRIVER={driver};SERVER={server};DATABASE={database};UID={usuario};PWD={senha}'
            conexao = pyodbc.connect(conn_str)
            # print("✅ Conexão SQL Server bem-sucedida!")
            module_logger.info("✅ Conexão SQL Server bem-sucedida!")

            return conexao
        except pyodbc.Error as e:
            # print("❌ Erro ao conectar ao banco:", e)
            module_logger.error(f"❌ Erro ao conectar ao banco: {e}")
            module_logger.error("❌ Verifique a conexão com a rede/VPN")
            raise pyodbc.Error(f"❌ Erro ao conectar sqlserver: {e}")

    def executar_update_insert_delete(self, conexao, query):
        """
        Executa uma query de UPDATE, INSERT ou DELETE e faz um commit usando uma conexão pyodbc.

        :param conexao: objeto de conexão pyodbc já aberto.
        :type conexao: pyodbc.Connection
        :param query: Instrução SQL de UPDATE ou INSERT.
        :type query: str

        :returns: Objeto de conexão pyodbc
        :rtype: pyodbc.Connection
        """

        if 'UPDATE' in query or 'update' in query or 'Update' in query:
            acao_executada = 'UPDATE'
        elif 'INSERT' in query or 'insert' in query or 'Insert' in query:
            acao_executada = 'INSERT'
        elif 'DELETE' in query or 'delete' in query or 'Delete' in query:
            acao_executada = 'DELETE'
            
        try:
            cursor = conexao.cursor()
            cursor.execute(query)
            conexao.commit()

            module_logger.info(f"✅ {acao_executada} executado com sucesso.")

            return True
        except pyodbc.Error as e:
            # print("❌ Erro ao executar o UPDATE:", e)
            module_logger.error(f"❌ Erro ao executar {acao_executada}: {e}")
            conexao.rollback()
            raise pyodbc.Error(f"❌ Erro ao executar {acao_executada}: {e}")
        finally:
            cursor.close()

    def atualizar_monitoramento(self, conexao, id_robo):
        """
        Atualiza o campo DATULTPRE da tabela de monitoramento com a data do ultimo processamento.

        :param conexao: objeto de conexão pyodbc já aberto.
        :type conexao: pyodbc.Connection
        :param id_robo: ID do robo na tabela NAGINTEGRA.NGMOND00
        :type id_robo: int
        """
        module_logger.info("Atualizando monitoramento...")
        query = f"""
                    SELECT *
                    FROM NAGINTEGRA.NGMOND00
                    WHERE 1=1
                    AND NOMSEV LIKE 'RPA%'
                    AND NUMSQC = {id_robo}
                """

        resultado_select = self.executar_select(conexao, query)

        if resultado_select:
            query_update = f"""
                            UPDATE NAGINTEGRA.NGMOND00 SET DATULTPRE = CURRENT_TIMESTAMP 
                            WHERE 1=1
                            AND NUMSQC = {id_robo}
                            """

            self.executar_update_insert_delete(conexao, query_update)

            module_logger.info("✅ TABELA DE MONITORAMENTO ATUALIZADA COM SUCESSO")

        else:
            module_logger.info(f"⚠️ Registro do robô com id {id_robo} não encontrado na tabela de monitoramento")
            module_logger.info("⚠️ Campo DATULTPRE não atualizado")
            module_logger.info("⚠️ Verifique se a conexão esta em produção")



import win32com.client
import win32gui
import win32con
import win32process
import win32api
import ctypes
import time
import os
import subprocess
import psutil
import re

from rpanagem import manipula_diretorios_arquivos
import logging
logger = logging.getLogger(__name__)
log = manipula_diretorios_arquivos.Arquivos()
module_logger = log.criar_logger_diario("./log", nome_arquivo_log="manipula_sap.log", logger=logger)


class SAP():
    """Faz conexões com o sistema SAP e minupula suas telas"""

    def __init__(self):
        pass

    def start_sap_logon(self, sap_path, sap_system, username, password, language):
        """
        Conectar e realiza login no SAP GUI Scripting (Automação Desktop).

        :param sap_path: Caminho onde esta instalado o executaval do sistema.
        :type sap_path: str
        :param sap_system: Opção de login no sistema (produção ou teste).
        :type sap_system: str
        :param username: Nome do usuário SAP.
        :type username: str
        :param password: Senha do usuário SAP.
        :type password: str
        :param language: Idioma do sistema.
        :type language: str

        :returns: se a conexão e o login for bem sucedido retorna a sessão no SAP que foi conectada.
        :rtype: str
        """
        if not os.path.exists(sap_path):
            module_logger.warning(f"❌ SAP Logon não encontrado em: {sap_path}")
            raise FileNotFoundError(f"❌ SAP Logon não encontrado em: {sap_path}")

        try:
            subprocess.Popen(sap_path)
            # print("SAP Logon iniciado...")
            module_logger.info("SAP Logon iniciado...")
            time.sleep(5)

            # Conectar ao SAP GUI
            SapGuiAuto = win32com.client.GetObject("SAPGUI")

            if not SapGuiAuto:
                module_logger.warning("ℹ️ SAP GUI não está rodando.")
                raise Exception("ℹ️ SAP GUI não está rodando.")

            application = SapGuiAuto.GetScriptingEngine
            connection = application.OpenConnection(sap_system, True)
            self.session = connection.Children(0)

            # print("✅ SAP Logon conectado.")
            module_logger.info("✅ SAP Logon conectado.")
        except Exception as e:
            # print("Erro ao conectar no SAP:", e)
            module_logger.error(f"❌ Erro ao conectar no SAP: {e}")
            self.fechar_sessao_sap(self.session)
            raise Exception(f"❌ Erro ao conectar no SAP: {e}")

        try:

            # print("Iniciando login no SAP...")
            module_logger.info("Iniciando login no SAP...")

            self.session.findById("wnd[0]/usr/txtRSYST-BNAME").text = username
            self.session.findById("wnd[0]/usr/pwdRSYST-BCODE").text = password
            self.session.findById("wnd[0]/usr/txtRSYST-LANGU").text = language
            self.session.findById("wnd[0]").sendVKey(0)
            # session.findById("wnd[0]/tbar[0]/btn[0]").press()  # Enter

            time.sleep(2)
            if "O nome ou a senha não está correto" in self.verificar_mensagem_status(self.session).lower():
                return None

            # Verificar se apareceu a janela de "logon múltiplo"
            if self.session.Children.Count > 1:
                for i in range(self.session.Children.Count):
                    wnd = self.session.Children(i)
                    if "logon múltiplo" in wnd.Text.lower():
                        module_logger.info("⚠️ Logon múltiplo detectado — login não realizado.")
                        self.fechar_sessao_sap(self.session)
                        return None

            # Verificar se o menu principal foi carregado (transação inicial)
            if not self.session.findById("wnd[0]/tbar[0]/okcd", False):
                # print("❌ Login não chegou à tela principal do SAP.")
                module_logger.info("❌ Login não chegou à tela principal do SAP.")
                return None
        
            # print("✅ Login realizado.")
            module_logger.info("✅ Login realizado.")

            return self.session
        except Exception as e:
            # print("Erro ao realizar o login no SAP:", e)
            module_logger.error(f"❌ Erro ao realizar o login no SAP: {e}")
            self.fechar_sessao_sap(self.session)
            raise Exception(f"❌ Erro ao realizar o login no SAP: {e}")

    def procurar_campos(self, session_sap, campos_ids=None):
        """
        Verifica se algum dos campos existem na sessao do sap.

        :param campos_ids: Lista de IDs de campos únicos na tela (ex: ["wnd[0]/usr/ctxtLFA1-STCD1"]).
        :type campos_ids: list

        :returns: True se a janela estiver aberta, False caso contrário.
        :rtype: bool
        """
        try:
            algum_campo_encontrado = False
            if campos_ids:
                for campo_id in campos_ids:
                    try:
                        session_sap.findById(campo_id)
                        algum_campo_encontrado = True
                        return algum_campo_encontrado
                    except:
                        continue
                return algum_campo_encontrado

        except Exception as e:
            # print(f"❌ Erro ao verificar os campos: {e}")
            module_logger.error(f"❌ Erro ao verificar os campos: {e}")
            raise Exception(f"❌ Erro ao verificar os campos: {e}")

        # Exemplo: Verificar pela existência de campos (ex: CNPJ ou IE)
        # campos_fiscais = [
        #     "wnd[0]/usr/ctxtLFA1-STCD1",  # CNPJ/CPF
        #     "wnd[0]/usr/ctxtLFA1-STCD2",  # Inscrição Estadual
        # ]

        # procurar_campos(session, campos_ids=campos_fiscais):

    def procurar_janela(self, session_sap, titulo_procurado, usar_regex=False):
        """
        Verifica se existe alguma janela ativa no SAP tem o título informado.
        Ou Verifica se algum dos campos únicos existe.

        :param session_sap: Sessão SAP ativa
        :type session_sap: win32com session
        :param titulo_procurado: Título da janela ou padrão regex. (ex: "Domicílio Fiscal").
        :type titulo_procurado: str
        :param usar_regex : Se True, trata titulo_procurado como expressão regular.
        :type usar_regex : bool

        :returns: True se a janela estiver ativa, False caso contrário.
        :rtype: bool
        """
        try:
            idx = 0
            while True:
                janela_id = f"wnd[{idx}]"
                try:
                    janela = session_sap.findById(janela_id)
                    titulo = janela.Text.strip()

                    if usar_regex:
                        if re.search(titulo_procurado, titulo, re.IGNORECASE):
                            module_logger.info(f'✅ Janela com regex encontrada: {titulo}')
                            return True
                    else:
                        if titulo_procurado.lower() in titulo.lower():
                            module_logger.info(f'✅ Janela encontrada: {titulo}')
                            return True

                    idx += 1
                except:
                    break  # Não há mais janelas
            return False
        except Exception as e:
            module_logger.error(f"❌ Erro ao verificar janelas SAP: {e}")
            # print(f"❌ Erro ao verificar janelas SAP: {e}")
            return False

    def verificar_mensagem_status(self, session_sap):
        """
        Verifica a última mensagem de status exibida no SAP GUI.

        :param session_sap: objeto Session (SAP GUI) já conectado.
        :type session_sap: win32com.client

        :returns: Mensagem de status atual
        :rtype: str
        """
        try:
            barra_status = session_sap.findById("wnd[0]/sbar")
            if barra_status:
                mensagem = barra_status.Text.strip()

                if mensagem:
                    # print(f"📣 Mensagem SAP: {mensagem}")
                    module_logger.info(f"📣 Mensagem SAP: {mensagem}")

                return mensagem
            else:
                module_logger.info(f"⚠️ Nenhuma mensagem SAP obtida.")
        except Exception as e:
            module_logger.error(f"❌ Erro ao acessar a barra de status: {e}")
            raise Exception(f"❌ Erro ao acessar a barra de status: {e}")

    def fechar_sessao_sap(self, session_sap):
        """
        Fecha uma sessão SAP aberta via SAP GUI Scripting.

        :param session_sap: Objeto da sessão SAP (Session)
        :type session_sap: win32com.client

        Exemplo de obtenção do objeto:
            sap_gui = win32com.client.GetObject("SAPGUI")
            app = sap_gui.GetScriptingEngine
            connection = app.Children(0)
            session = connection.Children(0)
        """
        try:
            if session_sap and session_sap.Info.IsLowSpeedConnection is False:
                # print("🔒 Fechando sessão SAP...")
                module_logger.info("🔒 Fechando sessão SAP...")

                session_sap.EndTransaction()
                session_sap = None  # limpa referência
                # print("✅ Sessão SAP encerrada com sucesso.")
                module_logger.info("✅ Sessão SAP encerrada com sucesso.")
            else:
                # print("⚠️ Sessão SAP inválida ou já encerrada.")
                module_logger.warning("⚠️ Sessão SAP inválida ou já encerrada.")
                return False
        except Exception as e:
            module_logger.error(f"❌ Erro ao fechar a sessão SAP: {e}")
            raise Exception(f"❌ Erro ao fechar a sessão SAP: {e}")


        try:
            # print("🧹 Encerrando processo saplogon.exe...")
            module_logger.info("🧹 Encerrando processo saplogon.exe...")

            for proc in psutil.process_iter(['name']):
                if proc.info['name'] and proc.info['name'].lower() == 'saplogon.exe':
                    proc.terminate()
                    proc.wait(timeout=5)
                    # print("✅ Processo SAP GUI finalizado.")
                    module_logger.info("✅ Processo SAP GUI finalizado.")
                    break
            else:
                # print("ℹ️ Processo saplogon.exe não encontrado.")
                module_logger.warning("ℹ️ Processo saplogon.exe não encontrado.")
                return False
        except Exception as e:
            # print(f"❌ Erro ao encerrar o processo SAP GUI: {e}")
            module_logger.error(f"❌ Erro ao encerrar o processo SAP GUI: {e}")
            raise Exception(f"❌ Erro ao encerrar o processo SAP GUI: {e}")

        return True

    def run_transaction(self, session_sap, transacao):
        """
        Acessa uma transação SAP via SAP GUI Scripting.

        :param session_sap: Objeto de conexão SAP
        :type session_sap: win32com.client
        :param transacao: Código da transação
        :type transacao: str

        """
        try:
            # Entra com a transação
            session_sap.findById("wnd[0]/tbar[0]/okcd").text = f"/n{transacao}"
            # session_sap.findById("wnd[0]/tbar[0]/btn[0]").press()
            session_sap.findById("wnd[0]").sendVKey(0)

            # Espera um pouco para a transação carregar (opcional)
            time.sleep(1)

            # print(f"Transação {transacao} acessada com sucesso.")
            module_logger.info(f"✅ Transação {transacao} acessada com sucesso.")

        except Exception as e:
            # print(f"Erro ao executar a transação {transacao}: {e}")
            module_logger.error(f"❌ Erro ao executar a transação {transacao}: {e}")
            raise Exception(f"❌ Erro ao executar a transação {transacao}: {e}")

    def anexar_sessao_sap(self, session_sap, wnd=0):
        """
        Recebe uma sessão SAP GUI, foca e traz a janela correspondente para o primeiro plano.

        :param session_sap: Objeto de conexão SAP
        :type session_sap: win32com.client
        """
        try:
            janela_id = f"wnd[{wnd}]"

            # Obtém o título da janela SAP atual
            titulo_janela = session_sap.findById(janela_id).Text
            module_logger.info(f"Janela encontrada {titulo_janela}.")

            # Procura a janela com esse título
            hwnd = win32gui.FindWindow(None, titulo_janela)

            if not hwnd or hwnd == 0:
                module_logger.info("Janela SAP não encontrada.")
                raise Exception("Janela SAP não encontrada.")

            # Traz a janela para o primeiro plano
            # win32gui.ShowWindow(hwnd, 5)
            # win32gui.SetForegroundWindow(hwnd)

            # Se estiver minimizada, restaura
            # win32gui.ShowWindow(hwnd, win32con.SW_RESTORE)
            win32gui.ShowWindow(hwnd, win32con.SW_MAXIMIZE)

            # Força o foco (workaround para SetForegroundWindow falhar em alguns casos)
            foreground_thread_id = win32process.GetWindowThreadProcessId(win32gui.GetForegroundWindow())[0]
            target_thread_id = win32process.GetWindowThreadProcessId(hwnd)[0]

            # Anexa os threads de entrada
            if foreground_thread_id != target_thread_id:
                ctypes.windll.user32.AttachThreadInput(foreground_thread_id, target_thread_id, True)

            # Traz para frente
            win32gui.SetForegroundWindow(hwnd)

            # foca
            try:
                win32gui.SetFocus(hwnd)
            except Exception as focus_error:
                # print("⚠️ SetFocus falhou, tentando fallback...")
                module_logger.warning("⚠️ SetFocus falhou, tentando fallback...")
                win32gui.BringWindowToTop(hwnd)

            # Desanexa os threads após o foco
            if foreground_thread_id != target_thread_id:
                ctypes.windll.user32.AttachThreadInput(foreground_thread_id, target_thread_id, False)

            # print("✅ Janela SAP trazida para frente com foco.")
            module_logger.info("✅ Janela SAP trazida para frente com foco.")

        except Exception as e:
            # print(f"❌ Erro ao tentar focar a janela SAP: {e}")
            module_logger.info(f"❌ Erro ao tentar focar a janela SAP: {e}")
